#include "MeshMainWindow.h"

#include <cmath>
#include <limits>
#include <iostream>
#include <qfiledialog.h>
#include <qmessagebox.h> 
#include <qlayout.h> 
#include <Route.h>

using namespace std;

namespace X3DTK {
namespace MESH {

MeshMainWindow::MeshMainWindow(QWidget* parent, const char* name, WFlags fl)
: MainWindow(parent, name, fl), _viewer00Focused(false), _viewer10Focused(false), _viewer01Focused(false), _viewer11Focused(false)
{
  _meshController = new MeshController();

  // setting controller
  meshViewer00->setController(_meshController);
  meshViewer10->setController(_meshController);
  meshViewer01->setController(_meshController);
  meshViewer11->setController(_meshController);

  RC::Route::connect(meshViewer00, &Viewer::doubleClicked, this, &MeshMainWindow::focusMeshViewer00);
  RC::Route::connect(meshViewer10, &Viewer::doubleClicked, this, &MeshMainWindow::focusMeshViewer10);
  RC::Route::connect(meshViewer01, &Viewer::doubleClicked, this, &MeshMainWindow::focusMeshViewer01);
  RC::Route::connect(meshViewer11, &Viewer::doubleClicked, this, &MeshMainWindow::focusMeshViewer11);
}

MeshMainWindow::~MeshMainWindow()
{
  _meshController->release();
  delete _meshController;
  
  RC::Route::disconnect(meshViewer00, &Viewer::doubleClicked, this, &MeshMainWindow::focusMeshViewer00);
  RC::Route::disconnect(meshViewer10, &Viewer::doubleClicked, this, &MeshMainWindow::focusMeshViewer10);
  RC::Route::disconnect(meshViewer01, &Viewer::doubleClicked, this, &MeshMainWindow::focusMeshViewer01);
  RC::Route::disconnect(meshViewer11, &Viewer::doubleClicked, this, &MeshMainWindow::focusMeshViewer11);
}

void MeshMainWindow::fileOpen()
{
  QString fileName = QFileDialog::getOpenFileName("", "X3D files (*.x3d *.X3D);;All files (*)", this);
  
  // In case of Cancel
  if (fileName.isEmpty())
    return;
    
  // Loads the file name and disabling the validation of the file by setting the 
  // 2nd parameter to false.
  _meshController->load(fileName);
  
  // QGLViewer settings.
  SFVec3f min = _meshController->getBBoxMin();
  SFVec3f max = _meshController->getBBoxMax();
  SFVec3f center = 0.5f*(max + min);
  SFVec3f radius = max - center;
  min = center - 2.0f*radius;
  max = center + 2.0f*radius;
  
  meshViewer00->setSceneBoundingBox(min.f_data(), max.f_data());
  meshViewer01->setSceneBoundingBox(min.f_data(), max.f_data());
  meshViewer10->setSceneBoundingBox(min.f_data(), max.f_data());
  meshViewer11->setSceneBoundingBox(min.f_data(), max.f_data());
  meshViewer00->showEntireScene();
  meshViewer01->showEntireScene();
  meshViewer10->showEntireScene();
  meshViewer11->showEntireScene(); 
  
  _meshController->notify();
}

void MeshMainWindow::fileSave()
{
  _meshController->save();
}

void MeshMainWindow::fileSaveAs()
{
  QString fileName = QFileDialog::getSaveFileName("", "X3D files (*.x3d *.X3D)", this);
  
  // In case of Cancel
  if (fileName.isEmpty())
    return;
  
  _meshController->saveAs(fileName);   
}

void MeshMainWindow::fileExit()
{
  _meshController->release();
  delete _meshController;

  exit(1);
}

void MeshMainWindow::focusMeshViewer00()
{
  _viewer00Focused = !_viewer00Focused;
  
  if (_viewer00Focused)
  {
    meshViewer10->hide();
    meshViewer01->hide();
    meshViewer11->hide();
    
    MainWindowLayout->setRowStretch(0, 2);
    MainWindowLayout->setColStretch(0, 2);
    MainWindowLayout->setRowStretch(1, 0);
    MainWindowLayout->setColStretch(1, 0);
  }  
  else
  {
    MainWindowLayout->setRowStretch(0, 1);
    MainWindowLayout->setColStretch(0, 1);
    MainWindowLayout->setRowStretch(1, 1);
    MainWindowLayout->setColStretch(1, 1);
    
    meshViewer10->show();
    meshViewer01->show();
    meshViewer11->show();
  }
}

void MeshMainWindow::focusMeshViewer10()
{
  _viewer10Focused = !_viewer10Focused;
  
  if (_viewer10Focused)
  {
    meshViewer00->hide();
    meshViewer01->hide();
    meshViewer11->hide();
    
    MainWindowLayout->setRowStretch(0, 2);
    MainWindowLayout->setColStretch(1, 2);
    MainWindowLayout->setRowStretch(1, 0);
    MainWindowLayout->setColStretch(0, 0);
  }  
  else
  {
    MainWindowLayout->setRowStretch(0, 1);
    MainWindowLayout->setColStretch(1, 1);
    MainWindowLayout->setRowStretch(1, 1);
    MainWindowLayout->setColStretch(0, 1);
    
    meshViewer00->show();
    meshViewer01->show();
    meshViewer11->show();
  }
}

void MeshMainWindow::focusMeshViewer01()
{
  _viewer01Focused = !_viewer01Focused;
  
  if (_viewer01Focused)
  {
    meshViewer00->hide();
    meshViewer10->hide();
    meshViewer11->hide();
    
    MainWindowLayout->setRowStretch(1, 2);
    MainWindowLayout->setColStretch(0, 2);
    MainWindowLayout->setRowStretch(0, 0);
    MainWindowLayout->setColStretch(1, 0);
  }  
  else
  {
    MainWindowLayout->setRowStretch(1, 1);
    MainWindowLayout->setColStretch(0, 1);
    MainWindowLayout->setRowStretch(0, 1);
    MainWindowLayout->setColStretch(1, 1);
    
    meshViewer00->show();
    meshViewer10->show();
    meshViewer11->show();
  }
}

void MeshMainWindow::focusMeshViewer11()
{
  _viewer11Focused = !_viewer11Focused;
  
  if (_viewer11Focused)
  {
    meshViewer00->hide();
    meshViewer10->hide();
    meshViewer01->hide();
    
    MainWindowLayout->setRowStretch(1, 2);
    MainWindowLayout->setColStretch(1, 2);
    MainWindowLayout->setRowStretch(0, 0);
    MainWindowLayout->setColStretch(0, 0);
  }  
  else
  {
    MainWindowLayout->setRowStretch(1, 1);
    MainWindowLayout->setColStretch(1, 1);
    MainWindowLayout->setRowStretch(0, 1);
    MainWindowLayout->setColStretch(0, 1);
    
    meshViewer00->show();
    meshViewer10->show();
    meshViewer01->show();
  }
}

}
}
